/*---------------------------------------------------------------------------
				map_case - offline map reader for Age of Adventure
			  Copyright (c) 2000-2001 Interactive Broadcasting Ltd

	This program is free software: you can redistribute it and /or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation: either version 2 of the License, or (at your
	option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY: without even the implied warranty of MERCHANTABILITY
	or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
	for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307 USA

	Program written and compiled with Borland C++ Builder 5

	Original programming: 	Alan Lenton
	Additional programming:	Stanley Lenton

	Home website:				www.ibgames.net
---------------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include "age_map.h"

#include <cstdio>

using namespace std;

#include "location.h"
#include "main.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------


Graphics::TBitmap	*AgeMap::map_gfx = 0;

/*----------------------------------
	Construct a map from a given file
	Params:	File stream pointer
----------------------------------*/
AgeMap::AgeMap(FILE *file)
{
	Location	*loc;
	int	errors = 0;
	char	*buffer = new char[2048];

	loc_list.erase(loc_list.begin(),loc_list.end());

	if(fgets(title,80,file) == 0)
		MessageDlg("Empty file!",mtInformation,TMsgDlgButtons() << mbOK,0);
	else
	{
		title[strlen(title) - 1] = '\0';		// remove the '\n'

		while(fgets(buffer,2048,file) != 0)
		{
			loc = new Location(buffer);
			if(!loc->IsOK())
			{
				errors++;
				delete loc;
			}
			else
				loc_list[loc->Column() + 32 * loc->Row()] = loc;
		}
	}

	if(errors > 0)
	{
		sprintf(buffer,"%d locations had errors",errors);
		MessageDlg(buffer,mtInformation,TMsgDlgButtons() << mbOK,0);
	}
	else
	{
		sprintf(buffer,"%d locations loaded",loc_list.size());
		MainForm->Status->Panels->Items[0]->Text = buffer;
	}

	sprintf(buffer,"Looking at map of %s",title);
	MainForm->Caption = buffer;

	delete [] buffer;

	return;
}

AgeMap::~AgeMap()
{
	Clear();
}



/*-----------------------------------
	AgeMap::CheckDiagonalLink()
	Find out if there is an left/right
	location link in this cell.
	Params:	Grid col & row
				Tile size in pixels
	Returns:	Source rect left coord
-----------------------------------*/
int	AgeMap::CheckDiagonalLink(int GridCol,int GridRow,int tile_size)
{
	int	diag = 0;

	if(CheckExit(GridCol - 1,GridRow + 1,2)) 	// sw/ne link?
		diag = 5 * tile_size;

	if(CheckExit(GridCol + 1,GridRow - 1,32))
		diag = 5 * tile_size;

	if(CheckExit(GridCol - 1,GridRow - 1,8) || CheckExit(GridCol + 1,GridRow + 1,128))	// se/nw link?
	{
		if(diag == 0)
			diag = 6 * tile_size;
		else
			diag = 7 * tile_size;		// crossed diagonals!!
	}

	return(diag);
}

/*----------------------------------
	AgeMap::CheckExit()
	Find out if there is an exit
	in the specified direction
	Params:	The row and col coords
				The direction
	Returns:	TRUE if there is an exit
				else FALSE
----------------------------------*/
bool	AgeMap::CheckExit(int GridCol,int GridRow, int dir)
{
	LocList::iterator	iter;
	int	loc_no, exits;

	if((GridCol > 0) && (GridRow > 0))
	{
		loc_no = (GridRow / 2)*32 + GridCol / 2;
		if((loc_no >= 0) && (loc_no < 32 * 32))
		{
			if((iter = loc_list.find(loc_no)) == loc_list.end())		// Not a real location
				return(false);
			exits = ((*iter).second)->Exits();
			if((exits & dir) == dir)
				return(true);
		}
	}

	return(FALSE);
}

/*----------------------------------------------------
	AgeMap::CheckLeftRightLink()
	Find out if there is an left/rt location link in
	this cell.
	Params:	Grid col & row
				Tile size in pixels
	Returns:	Source rect left coord
----------------------------------------------------*/
int	AgeMap::CheckLeftRightLink(int GridCol,int GridRow,int tile_size)
{
	if(CheckExit(GridCol - 1,GridRow,4))
		return(4 * tile_size);
	if(CheckExit(GridCol + 1,GridRow,64))
		return(4 * tile_size);

	return(0);
}

/*--------------------------------------------------
	AgeMap::CheckUpDownLink()
	Find out if there is an up/down location link in
	this cell.
	Params:	Grid col & row
				Tile size in pixels
	Returns:	Source rect left coord
--------------------------------------------------*/
int	AgeMap::CheckUpDownLink(int GridCol,int GridRow,int tile_size)
{
	if(CheckExit(GridCol,GridRow - 1,16))
		return(3 * tile_size);
	if(CheckExit(GridCol,GridRow + 1,1))
		return(3 * tile_size);

	return(0);
}

/*----------------------------------
	AgeMap::Clear()
	Delete all the existing locations
	on the map and delete the list.
----------------------------------*/
void	AgeMap::Clear()
{
	Location	*loc;
	while(loc_list.size() > 0)
	{
		loc = (*(loc_list.begin())).second;
		loc_list.erase(loc_list.begin());
		delete loc;
	}
}

/*------------------------------------------
	AgeMap::DrawCell()
	Draw he specified cell on the map display
	Params:	The column of the grid
				The row of the gris
				The rectangle to draw on
				Tile size in pixels
------------------------------------------*/
void	AgeMap::DrawCell(int GridCol,int GridRow,TRect &rect,int tile_size)
{
	TRect	source;
	LocList::iterator	iter;
	int loc_num;

	source.Left = 0;												// Default - unknown location

	// Figure out what we should draw in this cell...
	if((GridCol >= 0) && (GridRow >= 0))					// should always be the case, but...
	{
		if(IsPossibleLoc(GridCol,GridRow))
		{
			loc_num = (GridRow/2) * 32 + GridCol/2;
			if((iter = loc_list.find(loc_num)) != loc_list.end())	// Location exists
			{
				if(((*iter).second)->Notes() != 0)
					source.Left = 1 * tile_size;
				else
					source.Left = 2 * tile_size;
			}
		}
		else
		{
			if((GridCol % 2) == 1)
				source.Left = CheckUpDownLink(GridCol,GridRow,tile_size);
			if((GridRow % 2) == 1)
				source.Left = CheckLeftRightLink(GridCol,GridRow,tile_size);
			if(((GridRow % 2) == 0) && ((GridCol % 2) == 0))
				source.Left = CheckDiagonalLink(GridCol,GridRow,tile_size);
		}
	}

	// ...and then draw it!
	source.Right = source.Left + tile_size;
	source.Top = 0;
	source.Bottom = source.Top + tile_size;
	MainForm->Grid->Canvas->CopyRect(rect,map_gfx->Canvas,source);

	return;
}

/*-----------------------------
	AgeMap::IsPossibleLoc()
	Check to see if this cell
	holds locations or links
	Params:	The row and column
	Returns:	TRUE if its a loc
				FALSE if its a link
-----------------------------*/
bool	AgeMap::IsPossibleLoc(int grid_col,int grid_row)
{
	if(((grid_col % 2) == 1) && ((grid_row % 2) == 1))
		return(TRUE);
	else
		return(FALSE);
}

/*--------------------------
	AgeMap::LoadMapGfx()
	Load in the graphics used
	by the map
--------------------------*/
void	AgeMap::LoadMapGfx()
{
	map_gfx = new Graphics::TBitmap;
	map_gfx->LoadFromFile(".\\grafix\\newlink.bmp");
	return;
}

/*--------------------------------------------
	AgeMap::Name2Status()
	Write the name of the location the cursor
	is over into the status bar. If the cursor
	isn't over a location, clear the status bar
	Params:	The grid cell coordinates
--------------------------------------------*/
void	AgeMap::Name2Status(int cur_col,int cur_row)
{
	int loc_no = (cur_row/2) * 32 + cur_col/2;
	char	buffer[160];

	if(IsPossibleLoc(cur_col,cur_row))
	{
		LocList::iterator iter = loc_list.find(loc_no);
		if(iter != loc_list.end())		// yes - it's a location
		{
			sprintf(buffer,"Loc number %d: %s",(*iter).first,((*iter).second)->Name());
			MainForm->Status->Panels->Items[0]->Text = buffer;
		}
		else
			MainForm->Status->Panels->Items[0]->Text = "";
	}
	else
		MainForm->Status->Panels->Items[0]->Text = "";

	return;
}

/*-----------------------------
	Agemap::Save()
	Save this map out to disk
	Params:	File stream pointer
-----------------------------*/
void	AgeMap::Save(FILE *file)
{
	fprintf(file,"%s\n",title);

	for(LocList::iterator iter = loc_list.begin();iter != loc_list.end();iter++)
		((*iter).second)->Save(file);

	return;
}

/*-----------------------------------
	AgeMap::ShowNotes()
	Show the notes for this location
	Params:	The grid cell coordinates
-----------------------------------*/
void	AgeMap::ShowNotes(int cur_col,int cur_row,int X, int Y)
{
	if(IsPossibleLoc(cur_col,cur_row))
	{
		LocList::iterator iter = loc_list.find((cur_row/2) * 32 + cur_col/2);
		if(iter != loc_list.end())		// yes - it's a location
			((*iter).second)->ShowNotes(X, Y);
	}

	return;
}


