/*---------------------------------------------------------------------------
				map_case - offline map reader for Age of Adventure
			  Copyright (c) 2000-2001 Interactive Broadcasting Ltd

	This program is free software: you can redistribute it and /or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation: either version 2 of the License, or (at your
	option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY: without even the implied warranty of MERCHANTABILITY
	or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
	for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307 USA

	Program written and compiled with Borland C++ Builder 5

	Original programming: 	Alan Lenton
	Additional programming:	Stanley Lenton

	Home website:				www.ibgames.net
---------------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include "location.h"

#include <cstring>

using namespace std;

#include "main.h"
#include "notes.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------


/*-------------------------------------
	Construct a location from a line of
	text with colon delimited fields.
	The order is: column:row:exits:name:
	Params:	Pointer to the text
-------------------------------------*/
Location::Location(char *text)
{
	char	*token;

	if((token = strtok(text,":")) == 0)
		column = row = -1;
	else
	{
		column = atoi(token);
		if((token = strtok(0,":")) == 0)
			column = row = -1;
		else
		{
			row = atoi(token);
			if((token = strtok(0,":")) == 0)
				column = row = -1;
			else
			{
				exits =  atoi(token);
				if((token = strtok(0,":\n")) == 0)
					column = row = -1;
				else
				{
					strncpy(name,token,MAX_NAME);
					name[MAX_NAME - 1] =  '\0';
					if((token =  strtok(0,"\n")) == 0)
						notes = 0;
					else
					{
						notes = new char[1280];		// cut a bit of slack :)
						strcpy(notes,token);
					}
				}
			}
		}
	}
}

Location::~Location()
{
	delete [] notes;
}


/*----------------------------
	Location::Save()
	Save this location to file
	Params:	Ptr to file stream
----------------------------*/
void	Location::Save(FILE *file)
{
	fprintf(file,"%d:%d:%d:%s",column,row,exits,name);
	if(notes == 0)
		fprintf(file,"\n");
	else
		fprintf(file,":%s\n",notes);

	return;
}

/*---------------------------------
	Location::ShowNotes()
	Show the notes for this location
  Params: X and Y point on the Main
  Form where the mouse was clicked
  for this location
---------------------------------*/
void	Location::ShowNotes(int X, int Y)
{
	MainForm->Status->Panels->Items[1]->Text = "";
	char	*token,temp[1280];
	String str, str2="";
	String str1 = "\n";
	int len, j = 1;

	MainForm->StaticText->Visible = false;
	MainForm->StaticText->Caption = "";
	NotesForm->NotesMemo->Clear();
	NotesForm->SetLoc(this);
	NotesForm->Caption = name;
	NotesForm->Top = MainForm->Top + 52;
	NotesForm->Left = MainForm->Left;
	if(notes != 0)
	{
		strcpy(temp,notes);
		token = strtok(temp,"~\n\r");
		while(token != 0)
		{
			NotesForm->NotesMemo->Lines->Add(token);
			str2 += token; 					// put notes into str2
			token = strtok(0,"~\n");
		}
	}
	else NotesForm->Show();
	if(notes!=0)    							// now put notes and name into the static Text
	{               							// if there are notes
		str = NotesForm->Caption;	   	// put the name into str
		len = str.Length();
		if(len > 39)    						// make it fit the width of the static text
		{
			j++;          						// count the number of lines the name makes
			str.Insert(str1,40);  			// place a CR at the end of each line
		}
		str += str1;   						// add a final CR to name

		// now deal with the notes and make them into lines to fit the static text
		len = str2.Length();
		if(len > 39)
		{
			for(int i = 40; i < len; i+=40)
			{
				j++;
				for(int k = i; k>0; k--)	// Don't want to split words
				{                        	// so go back to previous space to place CR
					if(str2[k] == ' ')
					{
						str2.Insert(str1,k);
						i = k;
						break;
					}
				}
			}
		}

		j++; // make sure the height of the static text is sufficient
		str += str2;
		MainForm->Status->Panels->Items[1]->Text = "Click Here to Edit the Selected Location's Notes";
		MainForm->StaticText->Caption = str;

		// Make the text appear to the left or right of the location
		// as appropriate to show the notes
		if(X >= 374)
			MainForm->StaticText->Left = X-300;
		else
			MainForm->StaticText->Left = X;
		MainForm->StaticText->Top = Y;
		MainForm->StaticText->Height = j *23;

		// Stop the static text appearing when a menu item is clicked
		if(X == 0 && Y == 0)
			MainForm->StaticText->Visible = false;
		else
			MainForm->StaticText->Visible = true;
	}
	NotesForm->IsClean(true);

	return;
}

/*-----------------------------------
	Location::UpdateNotes()
	Update the notes for this location
	from the entry on the notes form
-----------------------------------*/
void	Location::UpdateNotes()
{
	if(notes == 0)
		notes = new char[1280];

	strcpy(notes,NotesForm->NotesMemo->Text.c_str());
	if(notes[0] == '\0')
	{
		delete [] notes;
		notes = 0;
	}
	else
	{
		for(int count = 0;notes[count] != '\0';count++)
		{
			if((notes[count] ==  '\n') || (notes[count] ==  '\r'))
				notes[count] = '~';
		}
	}

	NotesForm->IsClean(true);
	MainForm->Grid->Refresh();
	MainForm->IsClean(false);

	return;
}



